// -*- C++ -*-
/*!
 * @file  ODERTM.cpp
 * @brief ODE RTM Component
 * @date $Date$
 *
 * $Id$
 */

#include "ODERTM.h"
#include "ODEfunction.h"
#include <ode/ode.h>                // ODEpwb_[t@C
#include <drawstuff/drawstuff.h>    // `pwb_[t@C
#include <math.h>
#ifdef _MSC_VER
#pragma warning(disable:4244 4305)  // for VC++, no precision loss complaints
#endif

// Module specification
// <rtc-template block="module_spec">
static const char* odertm_spec[] =
  {
    "implementation_id", "ODERTM",
    "type_name",         "ODERTM",
    "description",       "ODE RTM Component",
    "version",           "0.1",
    "vendor",            "Kyouzyu",
    "category",          "TEST",
    "activity_type",     "SPORADIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    "exec_cxt.periodic.rate", "1.0",
    // Configuration variables
    "conf.default.m_config", "0",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
ODERTM::ODERTM(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_TorqueIn("Torque", m_Torque),
    m_DesFlagIn("DesFlag", m_DesFlag),
    m_BodyIn("Body", m_Body),
    m_RotOut("Rot", m_Rot),
    m_PosOut("Pos", m_Pos),
    m_AngleOut("Angle", m_Angle),
    m_GyroOut("Gyro", m_Gyro),
	m_PreOut("Pre", m_Pre),
    m_TouchOut("Touch", m_Touch)


    // </rtc-template>
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("Torque", m_TorqueIn);
  registerInPort("DesFlag", m_DesFlagIn);
  registerInPort("Body", m_BodyIn);
  
  // Set OutPort buffer
  registerOutPort("Rot", m_RotOut);
  registerOutPort("Pos", m_PosOut);
  registerOutPort("Angle", m_AngleOut);
  registerOutPort("Gyro", m_GyroOut);
  registerOutPort("Pre", m_PreOut);
  registerOutPort("Touch", m_TouchOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

/*!
 * @brief destructor
 */
ODERTM::~ODERTM()
{
}



RTC::ReturnCode_t ODERTM::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("m_config", m_config, "0");
  
  // </rtc-template>
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t ODERTM::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ODERTM::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ODERTM::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t ODERTM::onActivated(RTC::UniqueId ec_id)
{
	fri = m_config;
  return RTC::RTC_OK;
}


RTC::ReturnCode_t ODERTM::onDeactivated(RTC::UniqueId ec_id)
{
	SimExit();
  return RTC::RTC_OK;
}


RTC::ReturnCode_t ODERTM::onExecute(RTC::UniqueId ec_id)
{
	if(this->m_BodyIn.isNew())
	{
		m_BodyIn.read();
		int length = m_Body.data.length();
		SimInit(&(m_Body.data[0]),length);
	}
	m_Rot.data.length(num_body*9);
	const dReal *rot;
	for(int i = 0;i < num_body;i++)
	{
		rot = dBodyGetRotation(body[i].body);
		for(int j = 0;j < 9;j++)
		{
			if(j < 3)m_Rot.data[i*9+j] = rot[j];
			else if(j < 6)m_Rot.data[i*9+j] = rot[j+1];
			else if(j < 9)m_Rot.data[i*9+j] = rot[j+2];
		}
	}
	m_RotOut.write();

	m_Pre.data.length(num_sensor);
	for(int i = 0;i < num_sensor;i++)
	{
		if(sensor[i].sp < 3)
		{
			m_Pre.data[i] = sensor[i].feedback.f1[sensor[i].sp];
		}
		else
		{
			m_Pre.data[i] = sensor[i].feedback.t1[sensor[i].sp-3];
		}
	}
	m_PreOut.write();
	

	m_Gyro.data.length(num_gyro);
	const dReal *qua;
	for(int i = 0;i < num_gyro;i++)
	{
		qua = dBodyGetQuaternion(body[gyro_sensor[i].num].body);
		if(gyro_sensor[i].sp == 0)
		{
			m_Gyro.data[i] = fabs(2*acos(qua[0]))*dir(qua[1]);
		}
		else if(gyro_sensor[i].sp == 1)
		{
			m_Gyro.data[i] = fabs(2*acos(qua[0]))*dir(qua[2]);
		}
		else if(gyro_sensor[i].sp == 2)
		{
			m_Gyro.data[i] = fabs(2*acos(qua[0]))*dir(qua[3]);
		}
	}
	m_GyroOut.write();

	m_Touch.data.length(num_touch);
	for(int i = 0;i < num_touch;i++)
	{
		m_Touch.data[i] = touch_sensor[i].touch;
	}
	m_TouchOut.write();

	m_Pos.data.length(num_body*3);
	const dReal *pos;
	for(int i = 0;i < num_body;i++)
	{
		pos = dBodyGetPosition(body[i].body);
		for(int j = 0;j < 3;j++)
		{
			m_Pos.data[i*3+j] = pos[j];
		}
	}
	m_PosOut.write();

	m_Angle.data.length(num_joint);
	for(int i = 0;i < num_joint;i++)
	{
		if(joint[i].Jointtype == HINGE)
		{
			m_Angle.data[i] = dJointGetHingeAngle(joint[i].joint);
		}
		else if(joint[i].Jointtype == SLIDER)
		{
			m_Angle.data[i] = dJointGetSliderPosition(joint[i].joint);
		}
		else
		{
			m_Angle.data[i] = 0.;
		}
	}
	m_AngleOut.write();



	if(this->m_TorqueIn.isNew())
	{
		
		m_TorqueIn.read();
		int length = m_Torque.data.length();
		for(int i = 0;i < length;i++)
		{
			joint[i].torque = m_Torque.data[i];
		}
		Step(0.01);
	}
	if(this->m_DesFlagIn.isNew())
	{
		m_DesFlagIn.read();
		SimExit2();
		SimInit(&body_data[0],body_length);
	}
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t ODERTM::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ODERTM::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ODERTM::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ODERTM::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ODERTM::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void ODERTMInit(RTC::Manager* manager)
  {
    coil::Properties profile(odertm_spec);
    manager->registerFactory(profile,
                             RTC::Create<ODERTM>,
                             RTC::Delete<ODERTM>);
  }
  
};


