
#include <ode/ode.h>                // ODEpwb_[t@C
#include <drawstuff/drawstuff.h>    // `pwb_[t@C
#ifdef _MSC_VER
#pragma warning(disable:4244 4305)  // for VC++, no precision loss complaints
#endif

#include <string.h>

#define HINGE 0
#define SLIDER 1
#define FIX 2
	
typedef struct{
	dBodyID body;
	dGeomID geom;
}MyLink;

typedef struct{
	dJointID joint;
	int Jointtype;
	int Free;
	double torque;
}MyJoint;

typedef struct{
	dJointFeedback feedback;
	int num;
	int sp;
}MySensor;

typedef struct{
	int num1;
	int num2;
	int touch;
}MyTouch;

typedef struct{
	int num;
	int sp;
	double angle;
}MyGyro;

dWorldID world;
dBodyID apple;
dSpaceID space;
dGeomID ground;
dJointGroupID contactgroup;
MyLink *body;
MyJoint *joint;
double *body_data;
MyTouch *touch_sensor;
MyGyro *gyro_sensor;
int body_length;
int num_joint = 0;
int num_body = 0;
int num_sensor = 0;
int num_touch = 0;
int num_gyro = 0;
MySensor *sensor;

void SimInit();
void SimExit();
void Step(double st);
static void nearCallback(void *data, dGeomID o1, dGeomID o2);
double fri = dInfinity;


void CreateBox(MyLink *link, double *data)
{
	dMass mass;
	link->body = dBodyCreate(world);
	dMassSetZero(&mass);
	dMassSetBoxTotal(&mass,data[8], data[2], data[3], data[4]);
	dBodySetMass(link->body,&mass);
	link->geom = dCreateBox(space, data[2], data[3], data[4]);
	dGeomSetBody(link->geom, link->body);
	dBodySetPosition(link->body, data[5], data[6], data[7]);

}

void CreateCapsule(MyLink *link, double *data)
{
		dMass mass;
	link->body = dBodyCreate(world);
	dMassSetZero(&mass);
	dMassSetCapsuleTotal(&mass,data[8], int(data[7]), data[2], data[3]);
	dBodySetMass(link->body,&mass);
	link->geom = dCreateCapsule(space, data[2], data[3]);
	dGeomSetBody(link->geom, link->body);
	dBodySetPosition(link->body, data[4], data[5], data[6]);
}

void CreateSphere(MyLink *link, double *data)
{
		dMass mass;
	link->body = dBodyCreate(world);
	dMassSetZero(&mass);
	dMassSetSphereTotal(&mass,data[6], data[2]);
	dBodySetMass(link->body,&mass);
	link->geom = dCreateSphere(space, data[2]);
	dGeomSetBody(link->geom, link->body);
	dBodySetPosition(link->body, data[3], data[4], data[5]);
}

void CreateCylinder(MyLink *link, double *data)
{
		dMass mass;
	link->body = dBodyCreate(world);
	dMassSetZero(&mass);
	dMassSetCylinderTotal(&mass,data[8],int(data[7]), data[2],data[3]);
	dBodySetMass(link->body,&mass);
	link->geom = dCreateCylinder(space, data[2], data[3]);
	dGeomSetBody(link->geom, link->body);
	dBodySetPosition(link->body, data[4], data[5], data[6]);
}

void CreateHinge(MyJoint *jo, MyLink link1, MyLink link2, double *data)
{
	jo->joint = dJointCreateHinge(world, 0);
	dJointAttach(jo->joint, link1.body, link2.body);
	dJointSetHingeAnchor(jo->joint, data[4], data[5], data[6]);
	dJointSetHingeAxis(jo->joint, data[7], data[8], data[9]);
	jo->Free = int(data[10]);
	jo->Jointtype = HINGE;
	jo->torque = 0.;
}

void CreateFix(MyJoint *jo, MyLink link1, MyLink link2)
{
	jo->joint =  dJointCreateFixed(world, 0);
	dJointAttach(jo->joint, link1.body, link2.body);
	dJointSetFixed(jo->joint);
	jo->Jointtype = FIX;
}

void CreateSlider(MyJoint *jo, MyLink link1, MyLink link2, double *data)
{
	jo->joint = dJointCreateSlider(world, 0);
	dJointAttach(jo->joint, link1.body, link2.body);
	dJointSetSliderAxis(jo->joint, data[4], data[5], data[6]);
	jo->Jointtype = SLIDER;
	jo->torque = 0.;
}

void Create(double *data, int length)
{
	body_data = new double[length];
	for(int i=0;i < length;i++)body_data[i] = data[i];
	body_length = length;
	int joint_num = 0;
	int body_num = 0;
	int sensor_num = 0;
	int touch_num = 0;
	int gyro_num = 0;
	for(int i = 0;i < length;i+=0)
	{
		if(data[i]==1.)
		{
			if(data[i+1]==0.)
			{
				joint_num += 1;
				i += 11;
			}
			else if(data[i+1]==1.)
			{
				joint_num += 1;
				i += 7;
			}
			else if(data[i+1]==2.)
			{
				joint_num += 1;
				i += 4;
			}
		}

		else if(data[i] == 0.)
		{
			if(data[i+1] == 0.)
			{
				body_num += 1;
				i += 9;
			}
			else if(data[i+1] == 1.)
			{
				body_num += 1;
				i += 7;
			}
			else if(data[i+1] == 2.)
			{
				body_num += 1;
				i += 9;
			}
			else if(data[i+1] == 3.)
			{
				body_num += 1;
				i += 9;
			}
		}
		else if(data[i] == 2.)
		{
			sensor_num += 1;
			i += 3;
		}
		else if(data[i] == 3.)
		{
			touch_num += 1;
			i += 3;
		}
		else if(data[i] == 4.)
		{
			gyro_num += 1;
			i += 3;
		}
	}
	body = new MyLink[body_num];
	joint = new MyJoint[joint_num];
	sensor = new MySensor[sensor_num];
	touch_sensor = new MyTouch[touch_num];
	gyro_sensor = new MyGyro[gyro_num];

	body_num = 0;
	joint_num = 0;
	sensor_num = 0;
	touch_num = 0;
	gyro_num = 0;
	for(int i = 0;i < length;i+=0)
	{
		
		if(data[i] == 0.)
		{
			if(data[i+1] == 0.)
			{
				CreateBox(&body[body_num],&data[i]);
				body_num += 1;
				i += 9;
			}
			else if(data[i+1] == 1.)
			{
				CreateSphere(&body[body_num],&data[i]);
				body_num += 1;
				i += 7;
			}
			else if(data[i+1] == 2.)
			{
				CreateCylinder(&body[body_num],&data[i]);
				body_num += 1;
				i += 9;
			}
			else if(data[i+1] == 3.)
			{
				CreateCapsule(&body[body_num],&data[i]);
				body_num += 1;
				i += 9;
			}
		}
		else if(data[i]==1.)
		{
			if(data[i+1]==0.)
			{
				CreateHinge(&joint[joint_num],body[int(data[i+2])],body[int(data[i+3])],&data[i]);
				joint_num += 1;
				i += 11;
			}
			else if(data[i+1]==1.)
			{
				CreateSlider(&joint[joint_num],body[int(data[i+2])],body[int(data[i+3])],&data[i]);
				joint_num += 1;
				i += 7;
			}
			else if(data[i+1]==2.)
			{
				CreateFix(&joint[joint_num],body[int(data[i+2])],body[int(data[i+3])]);
				joint_num += 1;
				i += 4;
			}
		}
		else if(data[i] == 2.)
		{
			dJointSetFeedback(joint[int(data[i+1])].joint,&sensor[sensor_num].feedback);
			sensor[sensor_num].num = int(data[i+1]);
			sensor[sensor_num].sp = int(data[i+2]);
			sensor_num += 1;
			i += 3;
		}
		else if(data[i] == 3.)
		{
			touch_sensor[touch_num].num1 = int(data[i+1]);
			touch_sensor[touch_num].num2 = int(data[i+2]);
			touch_sensor[touch_num].touch = 0;
			touch_num += 1;
			i += 3;
		}
		else if(data[i] == 4.)
		{
			gyro_sensor[gyro_num].num = int(data[i+1]);
			gyro_sensor[gyro_num].sp = int(data[i+2]);
			gyro_sensor[gyro_num].angle = 0.;
			gyro_num += 1;
			i += 3;
		}
	}
	num_joint = joint_num;
	num_body = body_num;
	num_sensor = sensor_num;
	num_touch = touch_num;
	num_gyro = gyro_num;
}


void SimInit(double *data, int length)
{	
  dInitODE();
  world = dWorldCreate();                  // Ȇn
  space        = dHashSpaceCreate(0);
  contactgroup = dJointGroupCreate(0);
  ground       = dCreatePlane(space,0,0,1,0);
  dWorldSetGravity(world,0,0,-0.2);        // d͐ݒ
  dWorldSetCFM(world, 1e-3); // CFM̐ݒ (global CFM)
  dWorldSetERP(world, 0.9);  // ERP̐ݒ (global ERP)
  Create(data,length);
  /*body = new MyLink[2];
  double data1[9] = {0,0,0.2,0.7,0.2,0.0,0.0,0.2,10.0};
  double data2[9] = {0,0,0.2,0.7,0.2,0.2,10.0};
  CreateSphere(&body[0], data2);
  CreateBox(&body[1], data1);
  joint = new MyJoint[2];
  double data3[11] = {1,0,0,1,0,0.35,0.3,0,0,1,1};
  double data4[7] = {1,0,0,1,0,1,1};
  CreateSlider(&joint[0], body[int(data3[2])], body[int(data3[3])], data4);*/
  //CreateFix(&joint[0], body[int(data3[2])], body[int(data3[3])]);
}




void SimExit()
{
	dJointGroupDestroy (contactgroup);
	for(int i=0;i < num_body;i++)dGeomDestroy(body[i].geom);
	dSpaceDestroy(space);
	dWorldDestroy(world);
	delete[] body;
	delete[] joint;
	delete[] sensor;
	delete touch_sensor;
	delete body_data;
	dCloseODE();
}
void SimExit2()
{
	dJointGroupDestroy (contactgroup);
	for(int i=0;i < num_body;i++)dGeomDestroy(body[i].geom);
	dSpaceDestroy(space);
	dWorldDestroy(world);
	delete[] body;
	delete[] joint;
	delete[] sensor;
	dCloseODE();
}

static void nearCallback(void *data, dGeomID o1, dGeomID o2)
{
  //int o1i = int(o1);
  //int o2i = int(o2);
  dBodyID b1 = dGeomGetBody(o1), b2 = dGeomGetBody(o2);
  if (b1 && b2 && dAreConnectedExcluding(b1,b2,dJointTypeContact)) return;
  // if ((o1 != ground) && (o2 != ground)) return;

  static const int N = 20;
  dContact contact[N];
  int n = dCollide(o1,o2,N,&contact[0].geom,sizeof(dContact));
  if (n > 0) {
    for (int i=0; i<n; i++) {
      contact[i].surface.mode = dContactSoftERP | dContactSoftCFM;
      contact[i].surface.mu   = fri; //2.0;
      contact[i].surface.soft_erp = 0.9;
      contact[i].surface.soft_cfm = 1e-5;
	  dJointID c = dJointCreateContact(world,contactgroup,&contact[i]);
      dJointAttach(c,b1,b2);
    }
  }
  for(int i = 0;i < num_touch;i++)
  {
	  if(touch_sensor[i].num1 == 100)
	  {
		  if((int(o1) == int(body[touch_sensor[i].num2].geom))&&(int(o2) == int(ground)))touch_sensor[i].touch = 1;
		  else if((int(o2) == int(body[touch_sensor[i].num2].geom))&&(int(o1) == int(ground)))touch_sensor[i].touch = 1;
	  }
	  else if(touch_sensor[i].num2 == 100)
	  {
		  if((int(o1) == int(body[touch_sensor[i].num1].geom))&&(int(o2) == int(ground)))touch_sensor[i].touch = 1;
		  else if((int(o2) == int(body[touch_sensor[i].num1].geom))&&(int(o1) == int(ground)))touch_sensor[i].touch = 1;
	  }
	  else
	  {
		  if((b1 == body[touch_sensor[i].num1].body)&&(b2 == body[touch_sensor[i].num2].body))touch_sensor[i].touch = 1;
		  else if((b1 == body[touch_sensor[i].num2].body)&&(b2 == body[touch_sensor[i].num1].body))touch_sensor[i].touch = 1;
	  }
  }
}

void control()
{
	for(int i = 0;i < num_joint;i++)
	{
		if(joint[i].Jointtype == HINGE)
		{
			if(joint[i].Free == 1)
			{
				//dJointSetHingeParam(joint[i].joint,  dParamVel, joint[i].torque);
				//dJointSetHingeParam(joint[i].joint, dParamFMax, fMax);
				dJointAddHingeTorque(joint[i].joint,  joint[i].torque);
				
			}
		}
		if(joint[i].Jointtype == SLIDER)
		{
			//dJointSetSliderParam(joint[i].joint, dParamVel,  joint[i].torque);
			//dJointSetSliderParam(joint[i].joint, dParamFMax, fMax);
			dJointAddSliderForce(joint[i].joint, joint[i].torque);
		}
	}
}

void Step(double st)
{	
	for(int i = 0;i < num_touch;i++)touch_sensor[i].touch = 0;
	dSpaceCollide(space,0,&nearCallback); // Փˌo (collision detection)
	control();
	dWorldStep(world,st);
	dJointGroupEmpty(contactgroup);       // ڐG_O[v (empty jointgroup)
}

double dir(double k)
{
	if(k>=0.0)return(1.0);
	else return(-1.);
}

