#!/usr/bin/env python
# -*- Python -*-

import sys
import time
sys.path.append(".")

# Import RTM module
import OpenRTM_aist
import RTC

from numpy import *

# Import Service implementation class
# <rtc-template block="service_impl">

# </rtc-template>

# Import Service stub modules
# <rtc-template block="consumer_import">
# </rtc-template>


# This module's spesification
# <rtc-template block="module_spec">
SampleController_spec = ["implementation_id", "SampleController", 
		 "type_name",         "SampleController", 
		 "description",       "Sample Controller Component", 
		 "version",           "0.1", 
		 "vendor",            "Kyuzyu", 
		 "category",          "TEST", 
		 "activity_type",     "DataFlowComponent", 
		 "max_instance",      "1", 
		 "language",          "Python", 
		 "lang_type",         "SCRIPT",
		 ""]
# </rtc-template>

class SampleController(OpenRTM_aist.DataFlowComponentBase):
	def __init__(self, manager):
		OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)

		self._d_angle = RTC.TimedDoubleSeq(RTC.Time(0,0),[])
		self._angleIn = OpenRTM_aist.InPort("angle", self._d_angle, OpenRTM_aist.RingBuffer(8))
		self._d_feed = RTC.TimedDoubleSeq(RTC.Time(0,0),[])
		self._feedIn = OpenRTM_aist.InPort("feed", self._d_feed, OpenRTM_aist.RingBuffer(8))
		self._d_touch = RTC.TimedLongSeq(RTC.Time(0,0),[])
		self._touchIn = OpenRTM_aist.InPort("touch", self._d_touch, OpenRTM_aist.RingBuffer(8))
		self._d_gyro = RTC.TimedDoubleSeq(RTC.Time(0,0),[])
		self._gyroIn = OpenRTM_aist.InPort("gyro", self._d_gyro, OpenRTM_aist.RingBuffer(8))
		self._d_torque = RTC.TimedDoubleSeq(RTC.Time(0,0),[])
		self._torqueOut = OpenRTM_aist.OutPort("torque", self._d_torque, OpenRTM_aist.RingBuffer(8))
		self._d_des = RTC.TimedBoolean(RTC.Time(0,0),[])
		self._desOut = OpenRTM_aist.OutPort("des", self._d_des, OpenRTM_aist.RingBuffer(8))		

		# Set InPort buffers
		self.registerInPort("angle",self._angleIn)
		self.registerInPort("feed",self._feedIn)
		self.registerInPort("touch",self._touchIn)
		self.registerInPort("gyro",self._gyroIn)
		
		# Set OutPort buffers
		self.registerOutPort("torque",self._torqueOut)
		self.registerOutPort("des",self._desOut)
		self.step = 0.
		

		

		

		
		# Set service provider to Ports
		
		# Set service consumers to Ports
		
		# Set CORBA Service Ports
		

		# initialize of configuration-data.
		# <rtc-template block="init_conf_param">
		
		# </rtc-template>


		 
	def onInitialize(self):
		# Bind variables and configuration variable
		
		return RTC.RTC_OK


	
	#def onFinalize(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#def onStartup(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#def onShutdown(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#def onActivated(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#def onDeactivated(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	def onExecute(self, ec_id):
	    if self._angleIn.isNew():
                kp = 10.0
                PI = 3.141592
                inport = self._angleIn.read()
                self._d_torque.data = []
                for i in range(0, 4):
                    target = 40 * PI / 180 * sin(self.step + i * 120 * 3.14 / 180)-inport.data[i]
                    self._d_torque.data.append(kp*target)
                self.step += 0.01
                self._torqueOut.write()
            """if self._feedIn.isNew():
                inport = self._feedIn.read()
                print inport.data"""
            if self._touchIn.isNew():
                inport = self._touchIn.read()
                print inport.data
            """if self._gyroIn.isNew():
                inport = self._gyroIn.read()
                print inport.data"""
            if self.step > 5:
                self._d_des.data = True
                self._desOut.write()
            if self.step > 5:
                self.step = 0.
	    return RTC.RTC_OK
	
	#def onAborting(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#def onError(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#def onReset(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#def onStateUpdate(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#def onRateChanged(self, ec_id):
	#
	#	return RTC.RTC_OK
	



def MyModuleInit(manager):
    profile = OpenRTM_aist.Properties(defaults_str=SampleController_spec)
    manager.registerFactory(profile,
                            SampleController,
                            OpenRTM_aist.Delete)

    # Create a component
    comp = manager.createComponent("SampleController")



def main():
	mgr = OpenRTM_aist.Manager.init(len(sys.argv), sys.argv)
	mgr.setModuleInitProc(MyModuleInit)
	mgr.activateManager()
	mgr.runManager()

if __name__ == "__main__":
	main()

